<?php

/**
 * @file
 * Contains \Drupal\admin_status\Plugin\AdminStatus\CoreStatusReport.
 */

namespace Drupal\admin_status\Plugin\AdminStatus;

use Drupal\Core\Form\FormStateInterface;

/**
 * Provides a System Report Error message.
 *
 * @Plugin(
 *   id = "core_status_report",
 *   name = "Core Status Report",
 *   admin_permission = "administer admin status",
 * )
 */
class CoreStatusReport extends AdminStatusPluginBase {

  var $system_manager = null;

  /**
   * Get a description/config fields of the admin status.
   */
  public function description() {
    return (string)$this->t('This is the Core Status Report for Admin Status.');
  }

  /**
   * Builds a sub-form if needed to configure this plugin.
   *
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   * @param array $configValues
   *   An array of saved configuration values for this plugin.
   * @return array
   *   A form array with any configuration form elements needed to configure
   *   this plugin.
   */
  public function configForm(array $form,
                             FormStateInterface $form_state,
                             array $configValues) {
    $form['message_type'] = array(
      '#type' => 'checkboxes',
      '#options' => array('error' => $this->t('Errors'), 'warning' => $this->t('Warnings')),
      '#title' => $this->t('Types of Core Status Report Errors/Warnings to show'),
      '#default_value' => empty($configValues['message_type']) ? '' : $configValues['message_type'],
    );

    return $form;
  }

  /**
   * Validates the plugin's configuration form values.
   *
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   * @param array $configValues
   *   An array of saved configuration values for this plugin.
   */
  public function configValidateForm(array $form,
                                     FormStateInterface $form_state,
                                     array $configValues) {
    // No validation needed for the checkboxes since they can all be empty.
    return;
  }

  /**
   * Process submitted data and return configuration values to be saved.
   *
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   * @param array $configValues
   *   An array of saved configuration values for this plugin.
   * @return array
   *   An array of configuration values to be saved for this plugin.
   */
  public function configSubmitForm(array $form,
                                   FormStateInterface $form_state,
                                   array $configValues) {
    $my_config = $form_state->getValue(
      array('plugins', 'core_status_report', 'config'));
    return $my_config;
  }

  /**
   * Return the drupal_set_message status for a given System Report severity.
   *
   * @param $severity
   * @return string
   */
  protected function translateSeverityToStatus($severity) {
    $system_manager = $this->system_manager;
    $status = '';
    switch ($severity) {
      case $system_manager::REQUIREMENT_WARNING:
        $status = 'warning';
        break;
      case $system_manager::REQUIREMENT_ERROR:
        $status = 'error';
        break;
    }
    return $status;
  }

  /**
   * Return a status message to display.
   *
   * @return array
   *  An array of message and status for the Admin Status module to present.
   */
  public function message(array $configValues) {
    $this->system_manager = \Drupal::service('system.manager');
    $system_manager = $this->system_manager;
    $requirements = $system_manager->listRequirements();
    $options = array_filter($configValues['message_type']);
    $systemSeverities = array();
    foreach ($options as $option) {
      switch ($option) {
        case 'warning':
          $systemSeverities[] = $system_manager::REQUIREMENT_WARNING;
          break;
        case 'error':
          $systemSeverities[] = $system_manager::REQUIREMENT_ERROR;
          break;
      }
    }
    $messageParts = array();
    $renderer = \Drupal::service('renderer');
    foreach ($requirements as $requirement) {
      if (isset($requirement['severity'])) {
        if (in_array($requirement['severity'], $systemSeverities)) {
          $severity = $requirement['severity'];
          unset($requirement['severity']);
          $renderArray = array('#theme' => 'status_report', '#requirements' => array($requirement));
          $renderedOutput = $renderer->renderPlain($renderArray);
          $messageParts[] = array('message' => $renderedOutput, 'status' => $this->translateSeverityToStatus($severity));
        }
      }
    }
    return $messageParts;
  }

}
